/*
 * Hydrogen
 * Copyright(c) 2002-2003 by Alex >Comix< Cominu [comix@users.sourceforge.net]
 *
 * http://hydrogen.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: Hydrogen.h,v 1.33 2003/02/22 13:07:48 comix Exp $
 *
 */
#ifndef HYDROGEN_H
#define HYDROGEN_H

#include "Song.h"
#include "Object.h"
#include "GenericDriver.h"
#include "EngineListener.h"

// Engine mode
#define ENGINE_LIVE_MODE	1
#define ENGINE_SONG_MODE	2

// Engine state
#define UNINITIALIZED	1
#define INITIALIZED		2
#define PREPARED		3
#define READY			4
#define PLAYING		5

// forward
class AlsaMidiDriver;

//----------------------------------------------------------------------------
/**
 * Hydrogen Audio Engine
 */
class Hydrogen : public Object {
	public:
		virtual string getClassName() {	return "Hydrogen";	}

		/** Return the Hydrogen instance */
		static Hydrogen* getInstance();

		/** Destructor */
		~Hydrogen();

		/** Start the internal sequencer */
		int start();

		/** Stop the internal sequencer */
		void stop();

		/** Set current song */
		void setSong(Song *newSong);

		/** Return the current song */
		Song* getSong();
		void removeSong();

		void playNote(Note *note);

		float getMasterPeak_L();
		float getMasterPeak_R();
		void setMasterPeak_L(float value);
		void setMasterPeak_R(float value);

		unsigned long getTickPosition();

		void setNextPattern(int pos);

		Pattern* getCurrentPattern();
		void setCurrentPattern(Pattern *newPattern);

		uint getPatternPos();
		uint getPlayingNotes();

		void setEngineMode(int mode);
		int getEngineMode();

//		bool isPlaying();

		void restartDrivers();

		uint exportSong(string filename);

		uint getSongNotesQueue();

		GenericDriver* getAudioDriver();
		AlsaMidiDriver* getMidiDriver();

		int getState();

		void addEngineListener(EngineListener *newListener);

	private:
		static Hydrogen* instance;

		/** Constructor */
		Hydrogen();

};

#endif

