/*
 * Hydrogen
 * Copyright(c) 2002-2003 by Alex >Comix< Cominu [comix@users.sourceforge.net]
 *
 * http://hydrogen.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: LocalFileMng.h,v 1.13 2003/01/02 22:07:11 comix Exp $
 *
 */

#ifndef LFILEMNG_H
#define LFILEMNG_H

#include <iostream>
#include <fstream>
#include <vector>
using std::vector;
#include <dirent.h>
#include <string>
using std::string;

#include "Song.h"
#include "Object.h"
#include "PreferencesMng.h"


// forward declaration
class TiXmlNode;
class Note;
class Instrument;
class InstrumentList;
class Sequence;
class Pattern;
class Song;

//----------------------------------------------------------------------------
/**
 * Class for load and save local files.
 */
class LocalFileMng : public Object{
	public:
		virtual string getClassName() {	return "LocalFileMng";	}

		/** Costruttore. */
		LocalFileMng();

		/** Distruttore. */
		~LocalFileMng();

		/** Restituisce la lista delle canzoni presenti. */
		vector<string> listSongs();

		/** Restituisce la lista degli strumenti presenti. */
		vector<string> listInstruments();

		/** Loads a song from disk */
		Song* loadSong(string filename);

		/** Save the song */
		void saveSong(Song *song, string filename);

		/** Loads an instrument from disk */
		Instrument* loadInstrument(string filename);
};





//-----------------------------------------------------------------------------
/**
 * Read XML file of a song
 */
class SongReader : public Object {
	private:
		/** Dato un XmlNode restituisce un oggetto Pattern */
		Pattern* getPattern(TiXmlNode* pattern, InstrumentList* instrList);

		/** Dato un XmlNode restituisce un oggetto sequence */
		Sequence* getSequence(TiXmlNode* sequence, InstrumentList* instrList);

		/** Dato un XmlNode restituisce un oggetto note */
		Note* getNote(TiXmlNode* node, InstrumentList *instrList);

	public:
		virtual string getClassName() {	return "SongReader";	}

		/** Constructor */
		SongReader();

		/** Destructor */
		~SongReader();

		Song* readSong(string filename);
};




//-----------------------------------------------------------------------------
/**
 * Write XML file of a song
 */
class SongWriter : public Object {
	private:

	public:
		virtual string getClassName() {	return "SongWriter";	}

		/** Constructor */
		SongWriter();

		/** Destructor */
		~SongWriter();

		void writeSong(Song *song, string filename);
};





//-----------------------------------------------------------------------------
/**
 * Instrument Reader
 */
class InstrumentReader : public Object {
	private:

	public:
		virtual string getClassName() {	return "InstrumentReader";	}

		/** Constructor */
		InstrumentReader();

		/** Destructor */
		~InstrumentReader();

		Instrument* readInstrument(string filename);
};




//-----------------------------------------------------------------------------
/**
 * Instrument Writer
 */
class InstrumentWriter : public Object {
	public:
		virtual string getClassName() {	return "InstrumentWriter";	}

		/** Constructor */
		InstrumentWriter();

		/** Destructor */
		~InstrumentWriter();

		void writeInstrument(Instrument *instrument, string filename);


};


#endif //LFILEMNG_H




