/*
 * Hydrogen
 * Copyright(c) 2002-2003 by Alex >Comix< Cominu [comix@users.sourceforge.net]
 *
 * http://hydrogen.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: PreferencesMng.h,v 1.14 2003/02/16 23:33:18 comix Exp $
 *
 */

#ifndef PREFERENCES_MNG
#define PREFERENCES_MNG

#include <string>
using std::string;

#include <pwd.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <fstream>
#include <iostream>
#include <fstream>
#include <dirent.h>
#include <stdio.h>
#include <sys/stat.h>

#include "Object.h"


const float FALLOFF_SLOW = 	1.08;
const float FALLOFF_NORMAL=	1.1;
const float FALLOFF_FAST =	1.5;

//----------------------------------------------------------------------------
/**
 * Manager for User Preferences File (singleton)
 */
class PreferencesMng : public Object {
	private:
		static PreferencesMng *instance;

		/** Audio buffer size */
		uint bufferSize;

		/** Audio driver */
		string audioDriver;

		/** Audio sample rate */
		uint sampleRate;

		/** Use metronome? */
		bool useMetronome;

		/** Metronome volume */
		float metronomeVolume;

		string applicationFontFamily;
		int applicationFontPointSize;

		float mixerFalloffSpeed;

		/** directory of demo songs */
		string demoPath;

		/** Get user's home directory. */
		string getUserHome();

		/** Constructor */
		PreferencesMng();

		/** Create preferences directory */
		void createPreferencesDirectory();

		/** Create data directory */
//		void createDataDirectory();

	public:
		/** Return an instance of PreferencesMng */
		static PreferencesMng* getInstance();

		/** Destructor */
		~PreferencesMng();

		virtual string getClassName() {	return "PreferencesMng";	}

		/** Load the preferences file */
		void loadPreferences();

		/** Save the preferences file */
		void savePreferences();


		/** Return the audio buffer size */
		uint getBufferSize() {	return bufferSize;	}

		/** Set the audio buffer size */
		void setBufferSize(uint bufferSize) {	this->bufferSize = bufferSize;	}

		/** return the selected audio driver */
		string getAudioDriver() {	return audioDriver;	}

		/** Selected an audio driver */
		void setAudioDriver(string audioDriver) {	this->audioDriver = audioDriver;	}

		/** return sample rate */
		uint getSampleRate() {	return sampleRate;	}

		/** set sample rate */
		void setSampleRate(uint sampleRate) {	this->sampleRate = sampleRate;	}

		/** return if metronome is enabled */
		bool isMetronomeEnabled() {	return useMetronome;	}

		/** Set if the metronome must be active */
		void setUseMetronome(bool useMetronome) {	this->useMetronome = useMetronome;	}

		/** return the metronome volume */
		float getMetronomeVolume() {	return metronomeVolume;	}

		/** set the metronome volume */
		void setMetronomeVolume(float metronomeVolume) {	this->metronomeVolume = metronomeVolume;	}

		string getApplicationFontFamily() {	return applicationFontFamily;	}
		void setApplicationFontFamily(string family) {	applicationFontFamily = family;	}

		int getApplicationFontPointSize() {	return applicationFontPointSize;	}
		void setApplicationFontPointSize(int size) {	applicationFontPointSize = size;	}

		float getMixerFalloffSpeed() {	return mixerFalloffSpeed;	}
		void setMixerFalloffSpeed(float value) {	mixerFalloffSpeed = value;	}


		string getDemoPath() {	return demoPath;	}

};

#endif

